﻿using System.Collections.Generic;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Web.Configuration;
using Microsoft.AnalysisServices.AdomdClient;

namespace xamWebPivotGrid.Web
{
    [ServiceContract(Namespace = "")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class AdomdWebService
    {
        [OperationContract]
        public List<CubeData> GetCubes()
        {
            AdomdConnection conn = new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            List<CubeData> cubes = new List<CubeData>();

            foreach (CubeDef cube in conn.Cubes)
            {
                if (cube.Type == CubeType.Cube)
                {
                    cubes.Add(new CubeData(
                                  cube.Name,
                                  cube.Caption,
                                  cube.Description));
                }
            }

            return cubes;
        }

        [OperationContract]
        public List<MeasureData> GetMeasures(string cubeName)
        {
            AdomdConnection conn = new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            List<MeasureData> measures = new List<MeasureData>();

            foreach (Measure measure in cubeDef.Measures)
            {
                measures.Add(
                    new MeasureData
                    {
                        Caption = measure.Caption,
                        Description = measure.Description,
                        Name = measure.Name,
                        UniqueName = measure.UniqueName
                    });
            }

            return measures;
        }

        [OperationContract]
        public List<DimensionData> GetDimensions(string cubeName)
        {
            AdomdConnection conn = new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            List<DimensionData> dimensions = new List<DimensionData>();

            foreach (Dimension dimension in cubeDef.Dimensions)
            {
                if (dimension.DimensionType == DimensionTypeEnum.Measure)
                {
                    continue;
                }

                DimensionData dimensionData = new DimensionData
                {
                    Caption = dimension.Caption,
                    Description = dimension.Description,
                    Name = dimension.Name,
                    UniqueName = dimension.UniqueName,
                    Hierarchies = new List<HierarchyData>()
                };

                foreach (Hierarchy hierarchy in dimension.Hierarchies)
                {
                    HierarchyData hierarchyData = new HierarchyData
                    {
                        // TODO: AllMember = hierarchy.DefaultMember,
                        Caption = hierarchy.Caption,
                        DefaultMember = hierarchy.DefaultMember,
                        Description = hierarchy.Description,
                        Name = hierarchy.Name,
                        UniqueName = hierarchy.UniqueName,
                        Levels = new List<LevelData>()
                    };

                    foreach (Level level in hierarchy.Levels)
                    {
                        LevelData levelData = new LevelData
                        {
                            Caption = level.Caption,
                            Depth = level.LevelNumber,
                            Description = level.Description,
                            Name = level.Name,
                            UniqueName = level.UniqueName
                        };

                        hierarchyData.Levels.Add(levelData);
                    }

                    dimensionData.Hierarchies.Add(hierarchyData);
                }

                dimensions.Add(dimensionData);
            }

            return dimensions;
        }

        [OperationContract]
        public List<MemberData> GetMembersOfLevel(string cubeName, string dimensionName, string hierarchyName, string levelName)
        {
            AdomdConnection conn =
                new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            Dimension dimension = cubeDef.Dimensions[dimensionName];

            if (dimension == null)
            {
                return null;
            }

            Hierarchy hierarchy = dimension.Hierarchies[hierarchyName];

            if (hierarchy == null)
            {
                return null;
            }

            Level level = hierarchy.Levels[levelName];

            if (level == null)
            {
                return null;
            }

            List<MemberData> members = new List<MemberData>();

            MemberCollection levelMembers = level.GetMembers();
            foreach (Member member in levelMembers)
            {
                members.Add(
                    new MemberData
                    {
                        Caption = member.Caption,
                        ChildCount = (int)member.ChildCount,
                        LevelDepth = member.LevelDepth,
                        LevelName = member.LevelName,
                        Name = member.Name,
                        ParentUniqueName = member.Parent == null ? null : member.Parent.UniqueName,
                        UniqueName = member.UniqueName
                    });
            }

            return members;
        }

        [OperationContract]
        public List<MemberData> GetMembersOfMember(string cubeName, string dimensionName, string hierarchyName, string levelName, string parentMemberName)
        {
            AdomdConnection conn =
                new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            Dimension dimension = cubeDef.Dimensions[dimensionName];

            if (dimension == null)
            {
                return null;
            }

            Hierarchy hierarchy = dimension.Hierarchies[hierarchyName];

            if (hierarchy == null)
            {
                return null;
            }

            Level level = hierarchy.Levels[levelName];

            if (level == null)
            {
                return null;
            }

            MemberCollection levelMembers = level.GetMembers();

            Member parentMember = levelMembers.Find(parentMemberName);

            MemberCollection parentMemberChildren = parentMember.GetChildren();

            List<MemberData> members = new List<MemberData>();

            foreach (Member member in parentMemberChildren)
            {
                members.Add(
                    new MemberData
                    {
                        Caption = member.Caption,
                        ChildCount = (int)member.ChildCount,
                        LevelDepth = member.LevelDepth,
                        LevelName = member.LevelName,
                        Name = member.Name,
                        ParentUniqueName = member.Parent == null ? null : member.Parent.UniqueName,
                        UniqueName = member.UniqueName
                    });
            }

            return members;
        }

        [OperationContract]
        public GridData GetGridData(string commandText)
        {
            GridData gridData = new GridData { Axes = new List<AxisData>() };

            AdomdConnection conn = new AdomdConnection(WebConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);

            conn.Open();

            AdomdCommand command = new AdomdCommand(commandText, conn);

            CellSet cellSet = command.ExecuteCellSet();

            foreach (Axis axis in cellSet.Axes)
            {
                AxisData axisData = new AxisData { Positions = new List<PositionData>() };

                foreach (Position position in axis.Positions)
                {
                    PositionData positionData = new PositionData
                    {
                        Ordinal = position.Ordinal,
                        Members = new List<MemberData>()
                    };
                    foreach (Member member in position.Members)
                    {
                        MemberData memberData = new MemberData
                        {
                            Caption = member.Caption,
                            ChildCount = (int)member.ChildCount,
                            LevelDepth = member.LevelDepth,
                            LevelName = member.LevelName,
                            Name = member.Name,
                            ParentUniqueName = member.Parent == null ? null : member.Parent.UniqueName,
                            UniqueName = member.UniqueName
                        };
                        positionData.Members.Add(memberData);
                    }

                    axisData.Positions.Add(positionData);
                }

                gridData.Axes.Add(axisData);
            }

            gridData.Cells = new List<CellData>();

            foreach (Cell cell in cellSet.Cells)
            {
                CellData cellData = new CellData
                {
                    FormattedValue = cell.FormattedValue,
                    Value = cell.Value
                };

                gridData.Cells.Add(cellData);
            }

            return gridData;
        }
    }

    [DataContract]
    public class CubeData
    {
        public CubeData(string name, string caption, string description)
        {
            this.Name = name;
            this.Caption = caption;
            this.Description = description;
        }

        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string Description { get; set; }
    }

    [DataContract]
    public class MeasureData
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string UniqueName { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string Description { get; set; }
    }

    [DataContract]
    public class DimensionData
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string UniqueName { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string Description { get; set; }

        [DataMember]
        public List<HierarchyData> Hierarchies { get; set; }
    }

    [DataContract]
    public class HierarchyData
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string UniqueName { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string Description { get; set; }

        [DataMember]
        public string DefaultMember { get; set; }

        [DataMember]
        public string AllMember { get; set; }

        [DataMember]
        public List<LevelData> Levels { get; set; }
    }

    [DataContract]
    public class LevelData
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string UniqueName { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string Description { get; set; }

        [DataMember]
        public int Depth { get; set; }
    }

    [DataContract]
    public class MemberData
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string UniqueName { get; set; }

        [DataMember]
        public string Caption { get; set; }

        [DataMember]
        public string ParentUniqueName { get; set; }

        [DataMember]
        public string LevelName { get; set; }

        [DataMember]
        public int LevelDepth { get; set; }

        [DataMember]
        public int ChildCount { get; set; }
    }

    [DataContract]
    public class GridData
    {
        [DataMember]
        public List<AxisData> Axes { get; set; }

        [DataMember]
        public List<CellData> Cells { get; set; }
    }

    [DataContract]
    public class AxisData
    {
        [DataMember]
        public List<PositionData> Positions { get; set; }
    }

    [DataContract]
    public class PositionData
    {
        [DataMember]
        public List<MemberData> Members { get; set; }

        [DataMember]
        public int Ordinal { get; set; }
    }

    [DataContract]
    public class CellData
    {
        [DataMember]
        public object Value { get; set; }

        [DataMember]
        public string FormattedValue { get; set; }
    }
}
