﻿using System;
using System.ComponentModel;
using Infragistics.Olap.AdomdClient.AdomdWebService;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Base;
using Infragistics.Olap.Xmla;

namespace Infragistics.Olap.AdomdClient
{
    public class AdomdModelProvider : XmlaDataProvider
    {
        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="AdomdModelProvider"/> class.
        /// </summary>
        /// <param name="connection">The connection should be of type <see cref="XmlaConnection"/>.</param>
        public AdomdModelProvider(IOlapConnection connection)
            : base(connection)
        {
        }

        #endregion

        #region Overrides of XmlaModelProvider

        /// <summary>
        /// Occurs when asynchronous call for loading schema is completed.
        /// </summary>
        public override event EventHandler<LoadSchemaCompletedEventArgs> LoadSchemaCompleted;

        /// <summary>
        /// Occurs when asynchronous call for loading measures is completed.
        /// </summary>
        public override event EventHandler<AsyncCompletedEventArgs> LoadMeasuresCompleted;

        /// <summary>
        /// Occurs when asynchronous call for loading dimensions is completed.
        /// </summary>
        public override event EventHandler<AsyncCompletedEventArgs> LoadDimensionsCompleted;

        /// <summary>
        /// Occurs when asynchronous call for loading members is completed.
        /// </summary>
        public override event EventHandler<AsyncCompletedEventArgs> LoadMembersCompleted;

        /// <summary>
        /// Loads the schema asynchronously.
        /// </summary>
        public override void LoadSchemaAsync()
        {
            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetCubesCompleted += this.ClientGetCubesCompleted;

            client.GetCubesAsync();
        }

        private void ClientGetCubesCompleted(object sender, GetCubesCompletedEventArgs e)
        {
            OlapSchema schema = new OlapSchema();

            foreach (CubeData cube in e.Result)
            {
                AddCube(schema, new Cube(cube.Name, cube.Caption, cube.Description));
            }

            if (this.LoadSchemaCompleted != null)
            {
                this.LoadSchemaCompleted(this, new LoadSchemaCompletedEventArgs(schema, e.Error, e.Cancelled, e.UserState));
            }
        }

        /// <summary>
        /// Loads the dimensions in the cube asynchronously.
        /// </summary>
        /// <param name="cube">The cube to fill dimensions in.</param>
        public override void LoadDimensionsAsync(ICube cube)
        {
            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetDimensionsCompleted += this.ClientGetDimensionsCompleted;

            client.GetDimensionsAsync(cube.Name, cube);
        }

        private void ClientGetDimensionsCompleted(object sender, GetDimensionsCompletedEventArgs e)
        {
            Cube dimensionsCube = e.UserState as Cube;
            if (dimensionsCube == null)
            {
                return;
            }

            for (int i = 0; i < e.Result.Count; i++)
            {
                DimensionData dimensionData = e.Result[i];
                Dimension dimension = new Dimension(
                    dimensionData.Name, 
                    dimensionData.UniqueName, 
                    dimensionData.Caption,
                    dimensionData.Description);

                for (int j = 0; j < dimensionData.Hierarchies.Count; j++)
                {
                    HierarchyData hierarchyData = dimensionData.Hierarchies[j];
                    Hierarchy hierarchy = new Hierarchy(
                        hierarchyData.Name,
                        hierarchyData.UniqueName,
                        hierarchyData.Caption,
                        hierarchyData.Description,
                        hierarchyData.DefaultMember,
                        hierarchyData.AllMember,
                        dimension,
						string.Empty,
						HierarchyOrigin.SystemEnabled);

                    for (int k = 0; k < hierarchyData.Levels.Count; k++)
                    {
                        LevelData levelData = hierarchyData.Levels[k];
                        Level level = new Level(
                            levelData.Name,
                            levelData.Caption,
                            levelData.UniqueName,
                            levelData.Description,
                            levelData.Depth,
                            hierarchy);

                        AddLevel(hierarchy, level);
                    }

                    AddHierarchy(dimension, hierarchy);
                }

                AddDimension(dimensionsCube, dimension);
            }

            if (this.LoadDimensionsCompleted != null)
            {
                this.LoadDimensionsCompleted(this, new AsyncCompletedEventArgs(e.Error, e.Cancelled, e.UserState));
            }
        }

        /// <summary>
        /// Loads the measures in the cube asynchronously.
        /// </summary>
        /// <param name="cube">The cube to fill measures in.</param>
        public override void LoadMeasuresAsync(ICube cube)
        {
            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetMeasuresCompleted += this.ClientGetMeasuresCompleted;

            client.GetMeasuresAsync(cube.Name, cube);
        }

        private void ClientGetMeasuresCompleted(object sender, GetMeasuresCompletedEventArgs e)
        {
            Cube measuresCube = e.UserState as Cube;
            if (measuresCube == null)
            {
                return;
            }

            for (int i = 0; i < e.Result.Count; i++)
            {
                MeasureData measureData = e.Result[i];
                AddMeasure(
                    measuresCube,
                    new Measure(measureData.Name, measureData.UniqueName, measureData.Caption, measureData.Description, string.Empty, MeasureAggregator.Sum));
            }

            if (this.LoadMeasuresCompleted != null)
            {
                this.LoadMeasuresCompleted(this, new AsyncCompletedEventArgs(e.Error, e.Cancelled, e.UserState));
            }
        }

        /// <summary>
        /// Loads the members and add them as a children to the parent member.
        /// </summary>
        /// <param name="cube">The cube in which the members are.</param>
        /// <param name="member">The parent member.</param>
        public override void LoadMembersAsync(ICube cube, IMember member)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Loads the members and add them as a children to the parent member.
        /// </summary>
        /// <param name="cube">The cube in which the members are.</param>   
        /// <param name="filterMember">The parent member.</param>
        public override void LoadMembersAsync(ICube cube, IFilterMember filterMember)
        {
            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetMembersOfMemberCompleted += this.ClientGetMembersOfMemberCompleted;

            client.GetMembersOfMemberAsync(cube.Name, filterMember.Member.ParentLevel.ParentHierarchy.ParentDimension.Name, filterMember.Member.ParentLevel.ParentHierarchy.Name, filterMember.Member.ParentLevel.Name, filterMember.Member.Name, filterMember);
        }

        private void ClientGetMembersOfMemberCompleted(object sender, GetMembersOfMemberCompletedEventArgs e)
        {
            Member parentMember = e.UserState as Member;
            if (parentMember == null)
            {
                IFilterMember filterMember = e.UserState as IFilterMember;
                if (filterMember == null)
                {
                    return;
                }

                parentMember = filterMember.Member as Member;
                if (parentMember == null)
                {
                    return;
                }
            }

            ClearMemberChildren(parentMember);
            
            for (int i = 0; i < e.Result.Count; i++)
            {
                MemberData memberData = e.Result[i];
                Member member = new Member(
                    memberData.Name,
                    memberData.UniqueName,
                    memberData.Caption,
                    memberData.ParentUniqueName,
                    memberData.LevelName,
                    memberData.LevelDepth,
                    parentMember.ParentLevel.ParentHierarchy.Levels[parentMember.ParentLevel.Depth + 1],
                    memberData.ChildCount);

                AddMember(parentMember, member);
            }

            if (this.LoadMembersCompleted != null)
            {
                this.LoadMembersCompleted(this, new AsyncCompletedEventArgs(e.Error, e.Cancelled, e.UserState));
            }
        }

        /// <summary>
        /// Loads the members and add them as a children to the level.
        /// </summary>
        /// <param name="cube">The cube in which the members are.</param>
        /// <param name="level">The parent level.</param>
        public override void LoadMembersAsync(ICube cube, ILevel level)
        {
            AdomdWebServiceClient client = new AdomdWebServiceClient();

            client.GetMembersOfLevelCompleted += this.ClientGetMembersOfLevelCompleted;

            client.GetMembersOfLevelAsync(cube.Name, level.ParentHierarchy.ParentDimension.Name, level.ParentHierarchy.Name, level.Name, level);
        }

        private void ClientGetMembersOfLevelCompleted(object sender, GetMembersOfLevelCompletedEventArgs e)
        {
            Level levelToFill = e.UserState as Level;
            Member parentMember = null;
            if (levelToFill == null)
            {
                parentMember = e.UserState as Member;
                if (parentMember == null)
                {
                    IFilterMember filterMember = e.UserState as IFilterMember;
                    if (filterMember == null)
                    {
                        return;
                    }

                    parentMember = filterMember.Member as Member;
                    if (parentMember == null)
                    {
                        return;
                    }
                }
            }

            if (levelToFill != null)
            {
                ClearMemberChildren(levelToFill);
            }
            else
            {
                ClearMemberChildren(parentMember);
            }

            for (int i = 0; i < e.Result.Count; i++)
            {
                MemberData memberData = e.Result[i];
                Member member = new Member(
                    memberData.Name,
                    memberData.UniqueName,
                    memberData.Caption,
                    memberData.ParentUniqueName,
                    memberData.LevelName,
                    memberData.LevelDepth,
                    levelToFill,
                    memberData.ChildCount);

                if (levelToFill != null)
                {
                    AddMember(levelToFill, member);
                }
                else
                {
                    AddMember(parentMember, member);
                }
            }

            if (this.LoadMembersCompleted != null)
            {
                this.LoadMembersCompleted(this, new AsyncCompletedEventArgs(e.Error, e.Cancelled, e.UserState));
            }
        }

        #endregion
    }
}
